/*
Contributors: Liam T. Monson
License: BSD 2-Clause license which should be found in the "license.txt"
file in the root game directory.
*/
#ifndef CHESS_G_DATA_TEMP_H
#define CHESS_G_DATA_TEMP_H
#include <GLFW/glfw3.h>
#include <string>
#include <queue>
#include "chessGameDataTypes.hpp"
#include "networkData.hpp"
//globals
extern GLFWwindow *window;
extern struct chessGData GData;
extern GLuint texObjects[NUM_TEXTURES];
const char* const textureFiles[] =  {
	//main menu
	"Assets/ChessTitle.png",
	"Assets/MenuLocalPlay.png",
	"Assets/MenuNetworkPlay.png",
	"Assets/MenuClient.png",
	"Assets/MenuServer.png",
	//in-game menu
	"Assets/MenuGoToMainMenu.png",
	//tiles
	"Assets/WTile.png",
	"Assets/BTile.png",
	//highlights
	"Assets/S_W_Highlight.png",
	"Assets/S_B_Highlight.png",
	"Assets/MoveHighlight.png",
	//turn glows
	"Assets/WGlow.png",
	"Assets/BGlow.png",
	"Assets/LastMoveGlow.png",
	//move glows
	"Assets/ValidMoveGlow.png",
	"Assets/InvalidMove.png",
	//----pieces
	//white
	"Assets/WPawn.png",
	"Assets/WKnight.png",
	"Assets/WBishop.png",
	"Assets/WRook.png",
	"Assets/WQueen.png",
	"Assets/WKing.png",
	//black
	"Assets/BPawn.png",
	"Assets/BKnight.png",
	"Assets/BBishop.png",
	"Assets/BRook.png",
	"Assets/BQueen.png",
	"Assets/BKing.png",
};

//chess game data
struct chessGData{
	//config file data
	configData configD;	
	//tech infrastructure
	int windowW;
	int windowH;
	GLfloat VVNegX;
	GLfloat VVPosX;
	GLfloat VVNegY;
	GLfloat VVPosY;
	double cursorX;
	double cursorY;
	//cursor pixel coords translated to viewing volume coords
	GLfloat tCursX;
	GLfloat tCursY;
	//controls game turns
	bool whiteTurn;
	specialMoves sMoves;
	menuSelections menuSel;
	bool inGameMenuOpen;
	//holds data determining what menu or selection stage the game is in
	int controlContext;
	//selected tile coords
	tileIndex selectedTile;	
	tileIndex heldPieceLoc;
	//main board
	tile board[8][8];
	//board contrasted with main board to determine last moves
	tile lastBoard[8][8];
	//----------stores all unprocessed netPacks which provide game update input
	std::queue<networkPackage> netPacks;
	//main menu
	int menuState;
	menuBackground menuBG;
	//networking
	networkData networkD;
};

#endif
