/*
Contributors: Liam T. Monson
License: BSD 2-Clause license which should be found in the "license.txt"
file in the root game directory.
*/
#ifndef CHESS_GAME_DATA_TYPES_H
#define CHESS_GAME_DATA_TYPES_H
#include <enet/enet.h>
#include <GL/gl.h>
#define MAX_FPS 59
#define MAX_CONFIG_LINE_CHARS 200
#define MAX_CONFIG_LINE_TOKENS 2
#define NUM_TEXTURES 28
#define NOTHING -1

struct configData{
	int initWinWidth;
	int initWinHeight;
	std::string serverIP;
	enet_uint16 serverPort;
};

enum cEntity{
	//MAIN MENU
	CHESS_TITLE,
	LOCAL_PLAY,
	NETWORK_PLAY,
	NETWORK_CLIENT,
	NETWORK_SERVER,
	//IN-GAME MENU
	GO_TO_MAIN_MENU,
	//TILES
	WTILE,
	BTILE,
	//HIGHLIGHTS
	S_W_HIGHLIGHT,
	S_B_HIGHLIGHT,
	MOVE_HIGHLIGHT,
	//GLOWS
	WGLOW,
	BGLOW,
	//MOVE TYPES
	LAST_MOVE,
	VALID_MOVE,
	INVALID_MOVE,
	//PIECES
	//WHITE
	WPAWN,
	WKNIGHT,
	WBISHOP,
	WROOK,
	WQUEEN,
	WKING,
	//BLACK
	BPAWN,
	BKNIGHT,
	BBISHOP,
	BROOK,
	BQUEEN,
	BKING,
};

enum moveType{
	NOT_MOVE,
	//invalid moves
	CHECKS_OWN_KING,
	CAN_NOT_CASTLE_IN_CHECK,
	//valid moves
	NORMAL_MOVE,
	//special consideration valid moves
	PAWN_JUMP,
	EN_PASSANT,
	CASTLE,
	PROMOTION
};

enum inGameChoiceButton{
	QUEEN,
	ROOK,
	BISHOP,
	KNIGHT,
	ESCAPE,
};

//control context states
enum contextState{
	MAIN_MENU,
	GAME_TURN,
	HOLDING_PIECE,
	WAIT_SIDE_CHOICE,
	WAIT_PIECE_CHOICE,
	//checkmate or draw
	GAME_COMPLETION
};

//which menu/submenu is current. number represents index of depth (example: main menu is index 0, one level in is index 1)
enum menuState{
	MENU_MAIN_0,
	MENU_NETWORK_1
};

struct tile{
	bool lastMove;
	int color;
	int piece;
	int moveType;
};

struct tileIndex{
	int x;
	int y;
};

struct tileCoords{
	GLfloat x;
	GLfloat y;
};

enum networkRole{
	NON_NETWORK,
	CLIENT,
	SERVER
};

//stores special chess move state like if a king/rook has been moved preventing castle
//or a valid en passant location
struct specialMoves{
	tileIndex enPassant;
	//king moved booleans for determining if castling is viable
	bool WCastleValidKSide;
	bool WCastleValidQSide;
	bool BCastleValidKSide;
	bool BCastleValidQSide;
};

struct menuSelections{
	//main
	bool localPlay;
	bool networkPlay;
	//network
	bool client;
	bool server;
	//in-game menu
	bool exitToMainMenu;
};

struct menuBackground{
	tile menuBoard[8][8];
	tileCoords tileOffsets[8][8];
};

enum commandType{
	SELECT,
	DESELECT,
	KEYBOARD_PRESS,
	INVALID_INPUT
};

struct command{
	commandType type;
	tileIndex target;
	int choice;
};

#endif
