/*
Contributors: Liam T. Monson
License: BSD 2-Clause license which should be found in the "license.txt"
file in the root game directory.
*/
#include "gameLogic.hpp"
void initChessGData(struct chessGData &GData){
	//update values found in config.txt
	parseConfig(GData);
	//initializes game contexts
	GData.whiteTurn = true;
	GData.sMoves.enPassant.x = NOTHING;
	GData.sMoves.enPassant.y = NOTHING;
	GData.sMoves.WCastleValidKSide = true;
	GData.sMoves.WCastleValidQSide = true;
	GData.sMoves.BCastleValidKSide = true;
	GData.sMoves.BCastleValidQSide = true;
	GData.menuSel.localPlay = false;
	GData.menuSel.networkPlay = false;
	GData.menuSel.client = false;
	GData.menuSel.server = false;
	GData.menuSel.exitToMainMenu = false;
	GData.inGameMenuOpen = false;
	GData.controlContext = MAIN_MENU;
	GData.selectedTile.x = 4;
	GData.selectedTile.y = 4;
	GData.heldPieceLoc.x = NOTHING;
	GData.heldPieceLoc.y = NOTHING;
	initBoard(GData.board);
	initBoard(GData.menuBG.menuBoard);
	//copy board to lastBoard (meaning there are initially no last moves)
	copyBoard(GData.lastBoard, GData.board);
	initTileCoords(GData.menuBG.tileOffsets);
	initNetPacks(GData);
	GData.menuState = MENU_MAIN_0;
	//init network data
	GData.networkD.networkRole = NON_NETWORK;
	for(int i = 0; i < NETWORK_TEXT_MESSAGE_CHARS; ++i){
		GData.networkD.textMessage[i] = '\0';
	}
	initNetworking(GData, GData.networkD.networkRole);
}

void initBoard(struct tile board[8][8]){
	//loops to create non-null values
	for(int i = 0; i < 8; ++i){
		for(int j = 0; j < 8; ++j){
			//sets all lastMove to false, as there hasn't been a last move yet
			board[i][j].lastMove = false;
			//creates checkerboard pattern
			if((i + j) % 2 == 0){
				board[i][j].color = WTILE;
			}
			else{
				board[i][j].color = BTILE;
			}
			board[i][j].piece = NOTHING;	
			board[i][j].moveType = NOT_MOVE;	
		}
	}
	//sets propper piece starting positions
	//setup pawns
	for(int i = 0; i < 8; ++i){
		board[i][1].piece = WPAWN;
		board[i][6].piece = BPAWN;
	}
	//setup knights
	for(int i = 1; i < 7; i += 5){
		board[i][0].piece = WKNIGHT;
		board[i][7].piece = BKNIGHT;
	}
	//setup bishops
	for(int i = 2; i < 7; i += 3){
		board[i][0].piece = WBISHOP;
		board[i][7].piece = BBISHOP;
	}
	//setup rooks
	for(int i = 0; i < 8; i += 7){
		board[i][0].piece = WROOK;
		board[i][7].piece = BROOK;
	}
	//setup queens
		board[4][0].piece = WQUEEN;
		board[4][7].piece = BQUEEN;
	//setup kings	
		board[3][0].piece = WKING;
		board[3][7].piece = BKING;
}

void initTileCoords(struct tileCoords tileLocs[8][8]){
	for(int i = 0; i < 8; ++i){
		for(int j = 0; j < 8; ++j){
			tileLocs[i][j].x = i * 10 + 5;
			tileLocs[i][j].y = 70 - (j * 10) + 5;
		}
	}
}

void initNetPacks(chessGData &GData){
	//pop off all netPacks to make sure there's no residue commands from a previous game
	while(!GData.netPacks.empty()){
		GData.netPacks.pop();
	}
}

bool playerControlsSide(bool sideControl[2], bool whiteTurn){
	bool returnVal = false;
	if(whiteTurn && sideControl[0] || !whiteTurn && sideControl[1]){
		returnVal = true;
	}
	return returnVal;
}

tileIndex genTileIndex(int x, int y){
	tileIndex newIndex;
	newIndex.x = x; newIndex.y = y;
	return newIndex;
}

bool isWithinBoard(int x, int y){
	bool returnVal = false;
	if(x >= 0 && x < 8 && y >= 0 && y < 8){
		returnVal = true;
	}
	return returnVal;
}

bool isPiece(int piece){
	bool returnVal = false;
	if(piece >= WPAWN && piece <= BKING){
		returnVal = true;
	}
	return returnVal;
}

bool isHeldPiece(tileIndex heldPieceLoc, int x, int y){
	bool returnVal = false;
	if(heldPieceLoc.x == x && heldPieceLoc.y == y){
		returnVal = true;
	}
	return returnVal;
}

bool isWhite(int piece){
	bool returnVal = false;
	if(piece >= WPAWN && piece <= WKING){
		returnVal = true;
	}
	return returnVal;
}

bool isBlack(int piece){
	bool returnVal = false;
	if(piece >= BPAWN && piece <= BKING){
		returnVal = true;
	}
	return returnVal;
}

bool isTurnPiece(int piece, bool whiteTurn){
	bool returnVal = false;
	if(whiteTurn){
		returnVal = isWhite(piece);
	}
	else{
		returnVal = isBlack(piece);
	}
	return returnVal;
}

bool isEnemyPiece(int piece, bool whiteTurn){
	bool returnVal = false;
	if(whiteTurn){
		returnVal = isBlack(piece);
	}
	else{
		returnVal = isWhite(piece);
	}
	return returnVal;
}

bool isInCheck(struct tile board[8][8], struct specialMoves sMoves, bool whiteTurn){
	bool retVal = false;
	int king = (whiteTurn) ? WKING: BKING; 
	int x; int y;
	//find our king's index
	for(int i = 0; i < 8; ++i){
		for(int j = 0; j < 8; ++j){
			if(board[i][j].piece == king){
				x = i; 
				y = j;
			}
		}
	}
	retVal = isInDanger(board, sMoves, whiteTurn, x, y);
	return retVal;
}

bool isCheckMate(struct tile board[8][8], struct specialMoves sMoves, int controlContext, bool whiteTurn){
	bool returnVal = false;
	struct tile boardClone[8][8];
	copyBoard(boardClone, board);
	if(isInCheck(boardClone, sMoves, whiteTurn) && !sideHasMoves(boardClone, sMoves, controlContext, whiteTurn)){
		returnVal = true;
	}
	return returnVal;
}

bool isDraw(struct tile board[8][8], struct specialMoves sMoves, int controlContext, bool whiteTurn){
	bool returnVal = false;
	struct tile boardClone[8][8];
	copyBoard(boardClone, board);
	int turnPieces = 0;
	int enemyPieces = 0;
	//if the king isn't in check but the moving side has no valid moves
	if(!isInCheck(boardClone, sMoves, whiteTurn) && !sideHasMoves(boardClone, sMoves, controlContext, whiteTurn)){
		returnVal = true;
	}
	//count pieces on each side
	for(int i = 0; i < 8; ++i){
		for(int j = 0; j < 8; ++j){
			if(isTurnPiece(boardClone[i][j].piece, whiteTurn)){
				++turnPieces;	
			}
			if(isEnemyPiece(boardClone[i][j].piece, whiteTurn)){
				++enemyPieces;	
			}
		}
	}
	//if there is only the kings left
	if(turnPieces <= 1 && enemyPieces <= 1){
		returnVal = true;
	}
	return returnVal;
}

bool sideHasMoves(struct tile board[8][8], struct specialMoves sMoves, int controlContext, bool whiteTurn){
	bool returnVal = false;
	struct tile boardClone[8][8];
	copyBoard(boardClone, board);
	clearMoves(boardClone);
	for(int i = 0; i < 8; ++i){
		for(int j = 0; j < 8; ++j){
			if(isTurnPiece(boardClone[i][j].piece, whiteTurn)){
				createPieceMoves(boardClone, sMoves, genTileIndex(i, j), HOLDING_PIECE);
				if(hasMoves(boardClone, sMoves, controlContext, i, j)){
					returnVal = true;
				}
			}
			clearMoves(boardClone);
		}
	}
	return returnVal;
}

bool hasMoves(struct tile board[8][8], struct specialMoves sMoves, int controlContext, int x, int y){
	bool returnVal = false;
	struct tile boardClone[8][8];
	copyBoard(boardClone, board);
	clearMoves(boardClone);
	createPieceMoves(boardClone, sMoves, genTileIndex(x, y), HOLDING_PIECE);
	removeFriendlyCheckMoves(boardClone, sMoves, genTileIndex(x, y), controlContext);
	for(int i = 0; i < 8; ++i){
		for(int j = 0; j < 8; ++j){
			if(isValidMove(boardClone[i][j].moveType)){
				returnVal = true;
			}
		}
	}
	return returnVal;
}

bool isInDanger(struct tile board[8][8], struct specialMoves sMoves, bool whiteTurn, int x, int y){
	bool returnVal = false;
	bool inDanger[8][8];
	int piece;
	struct tile boardClone[8][8];
	copyBoard(boardClone, board);
	clearMoves(boardClone);
	for(int i = 0; i < 8; ++i){
		for(int j = 0; j < 8; ++j){
			piece = boardClone[i][j].piece;
			//generate moves
			if(isEnemyPiece(piece, whiteTurn)){
				createPieceMoves(boardClone, sMoves, genTileIndex(i, j), HOLDING_PIECE);
			}
			//translate moves into inDanger[8][8]
			for(int h = 0; h < 8; ++h){
				for(int k = 0; k < 8; ++k){
					if(isValidMove(boardClone[h][k].moveType)){
						inDanger[h][k] = true;
					}
					else{
						inDanger[h][k] = false;
					}
				}
			}
		}
	}
	//check to see if target index is inDanger
	returnVal = inDanger[x][y];
	return returnVal;
}

bool isValidMove(int moveType){
	bool returnVal = false;
	if(moveType >= NORMAL_MOVE){
		returnVal = true;
	}
	return returnVal;
}

bool isInvalidMove(int moveType){
	bool returnVal = false;
	if(moveType > NOT_MOVE && moveType < NORMAL_MOVE){
		returnVal = true;
	}
	return returnVal;
}

void setPlayerControl(bool sideControl[2], bool controlsWhite, bool controlsBlack){
	if(controlsWhite) sideControl[0] = true;
	if(controlsBlack) sideControl[1] = true;
}

void copyBoard(struct tile destBoard[8][8], struct tile sourceBoard[8][8]){
	for(int i = 0; i < 8; ++i){
		for(int j = 0; j < 8 ; ++j){
			destBoard[i][j] = sourceBoard[i][j];
		}
	}
}

void shiftStrip(struct tile board[8][8], int offset, int tarLine, bool shiftRow){
	//convert offset to modulus-friendly number
	if(offset < 0){
		offset += 8;
	}
	//-------------------------setup board with strip
	struct tile strip[8];
	int nIndex;
	//if we're moving a row
	if(shiftRow){
		for(int i = 0; i < 8; ++i){
			strip[i] = board[i][tarLine];
		}
		for(int i = 0; i < 8; ++i){
			//copy shifted strip tile to board
			nIndex = (i + offset) % 8;
			board[nIndex][tarLine] = strip[i];
		}
	}
	//if we're moving a column
	if(!shiftRow){
		for(int i = 0; i < 8; ++i){
			strip[i] = board[tarLine][i];
		}
		for(int i = 0; i < 8; ++i){
			//copy shifted strip tile to board
			nIndex = (i + offset) % 8;
			board[tarLine][nIndex] = strip[i];
		}
	}
}

void clearMoves(struct tile board[8][8]){
	for(int i = 0; i < 8; ++i){
		for(int j = 0; j < 8; ++j){
			board[i][j].moveType = NOT_MOVE;
		}
	}
}

void updateMoves(struct tile board[8][8], 
		struct specialMoves sMoves, 
		tileIndex heldPieceLoc, 
		int controlContext)
{
	bool whiteTurn = isWhite(board[heldPieceLoc.x][heldPieceLoc.y].piece);
	//only do move update if coming from a new context, 
	//otherwise the moves should all be the same
	static int lastControlContext = GAME_TURN;
	if(lastControlContext != controlContext){
		struct tile boardCopy[8][8];
		//sMovesCopy is modifiable state to test for checks
		struct specialMoves sMovesCopy;
		struct tileIndex tarLoc;
		int x = heldPieceLoc.x; int y = heldPieceLoc.y;
		int heldPiece = board[x][y].piece;
		//-------------------------reset all moves
		clearMoves(board);

		if(controlContext == HOLDING_PIECE){
		//-------------------------add valid moves
		createPieceMoves(board, sMoves, heldPieceLoc, controlContext);
		//-------------------------remove specially unavailable moves (like moving into check)
		removeFriendlyCheckMoves(board, sMoves, heldPieceLoc, controlContext);
		removeInvalidCastleMoves(board, sMoves, whiteTurn);
		}
	}
	//update last context to minimize move calculation
	lastControlContext = controlContext;
}

void removeFriendlyCheckMoves(struct tile board[8][8], struct specialMoves sMoves,
		 tileIndex heldPieceLoc, int controlContext)
{
	int ox = heldPieceLoc.x; int oy = heldPieceLoc.y;
	int heldPiece = board[ox][oy].piece;
	bool whiteTurn = isWhite(heldPiece);
	int king = (whiteTurn) ? WKING: BKING; 
	
	struct tile boardClone[8][8];
	copyBoard(boardClone, board);

	for(int i = 0; i < 8; ++i){
		for(int j = 0; j < 8; ++j){
			if(isValidMove(board[i][j].moveType)){
				//refresh variables
				copyBoard(boardClone, board);
				whiteTurn = isWhite(board[ox][oy].piece);
				//execute move then check to see if it would cause check
				if(executeMove(boardClone, sMoves, heldPieceLoc,
				genTileIndex(i, j), controlContext, whiteTurn))
				{
					whiteTurn = !whiteTurn;
				}
				if(board[i][j].moveType != PROMOTION){
					//if this move places the current side's king in check, disable it
					if(isInCheck(boardClone, sMoves, !whiteTurn)){
						board[i][j].moveType = CHECKS_OWN_KING;
					}
				}
				//special case because a promotion move doesn't toggle whiteTurn
				//until the player has chosen a promotion piece
				else if(board[i][j].moveType == PROMOTION){
					//if this move places the current side's king in check, disable it. 
					//Notice that we check for whiteTurn instead of !whiteTurn due to the special case
					if(isInCheck(boardClone, sMoves, whiteTurn)){
						board[i][j].moveType = CHECKS_OWN_KING;
					}
				}
			}
		}
	}
}

void createPieceMoves(struct tile board[8][8], struct specialMoves &sMoves,
		tileIndex heldPieceLoc, int controlContext)
{
	int heldPiece = board[heldPieceLoc.x][heldPieceLoc.y].piece;
	if(controlContext == HOLDING_PIECE){
		switch(heldPiece){
			case WPAWN:
			case BPAWN:
				createPawnMoves(board, sMoves.enPassant, heldPieceLoc);
				break;
			case WKNIGHT:
			case BKNIGHT:
				createKnightMoves(board, heldPieceLoc);
				break;
			case WBISHOP:
			case BBISHOP:
				createBishopMoves(board, heldPieceLoc);
				break;
			case WROOK:
			case BROOK:
				createRookMoves(board, heldPieceLoc);
				break;
			case WKING:
			case BKING:
				createKingMoves(board, sMoves, heldPieceLoc);
				break;
			case WQUEEN:
			case BQUEEN:
				createQueenMoves(board, heldPieceLoc);
				break;
			default:
				break;
		}
	}
}

void createPawnMoves(struct tile board[8][8], struct tileIndex &enPassant, tileIndex heldPieceLoc){
	int x = heldPieceLoc.x; int y = heldPieceLoc.y;
	int piece = board[x][y].piece;
	bool whiteTurn = isWhite(piece);
	//direction pawn can move is based on color
	int dir = (whiteTurn) ? 1:-1;
	int homeRow = (whiteTurn) ? 1:6;
	//normal 1 step forward
	createMove(board, x, y + dir, ONLY_NOTHING, NORMAL_MOVE, whiteTurn);
	//diagonal attacks
	createMove(board, x + 1, y + dir, ONLY_ENEMY, NORMAL_MOVE, whiteTurn);
	createMove(board, x - 1, y + dir, ONLY_ENEMY, NORMAL_MOVE, whiteTurn);
	//create valid 2 space jump moves
	if(y == homeRow && isValidMove(board[x][y + dir].moveType)){
		createMove(board, x, y + (dir * 2), ONLY_NOTHING, PAWN_JUMP, whiteTurn);
	}
	//-------------------------SPECIAL MOVE: EN PASSANT
	if(y + dir == enPassant.y && ((enPassant.x == x - 1) || enPassant.x == x + 1)){
		createMove(board, enPassant.x, y + dir, ONLY_NOTHING, EN_PASSANT, whiteTurn);
	}
	//-------------------------SPECIAL MOVE: PROMOTION
	//if the pawn is 1 row below the top or bottom (about to be promoted)
	//convert moves to pawn promotion (if this is true it doesn't interfere with enPassant)
	if((whiteTurn && y == 6) || (!whiteTurn && y == 1)){
		for(int i = 0; i < 8; ++i){
			for(int j = 0; j < 8; ++j){
				if((board[i][j].moveType == NORMAL_MOVE) || (board[i][j].moveType == EN_PASSANT)){
					board[i][j].moveType = PROMOTION;
				}
			}
		}
	}
}

void createKnightMoves(struct tile board[8][8], tileIndex heldPieceLoc){
	int x = heldPieceLoc.x; int y = heldPieceLoc.y;
	int piece = board[x][y].piece;
	bool whiteTurn = isWhite(piece);
	//set 8 possible knight moves
	createMove(board, x + 1, y + 2, ENEMY_OR_NOTHING, NORMAL_MOVE, whiteTurn);
	createMove(board, x + 1, y - 2, ENEMY_OR_NOTHING, NORMAL_MOVE, whiteTurn);
	createMove(board, x - 1, y + 2, ENEMY_OR_NOTHING, NORMAL_MOVE, whiteTurn);
	createMove(board, x - 1, y - 2, ENEMY_OR_NOTHING, NORMAL_MOVE, whiteTurn);
	createMove(board, x + 2, y + 1, ENEMY_OR_NOTHING, NORMAL_MOVE, whiteTurn);
	createMove(board, x + 2, y - 1, ENEMY_OR_NOTHING, NORMAL_MOVE, whiteTurn);
	createMove(board, x - 2, y + 1, ENEMY_OR_NOTHING, NORMAL_MOVE, whiteTurn);
	createMove(board, x - 2, y - 1, ENEMY_OR_NOTHING, NORMAL_MOVE, whiteTurn);
}

void createBishopMoves(struct tile board[8][8], tileIndex heldPieceLoc){
	createStreakMove(board, heldPieceLoc, 1, 1);
	createStreakMove(board, heldPieceLoc, -1, -1);
	createStreakMove(board, heldPieceLoc, -1, 1);
	createStreakMove(board, heldPieceLoc, 1, -1);
}

void createRookMoves(struct tile board[8][8], tileIndex heldPieceLoc){
	int x = heldPieceLoc.x; int y = heldPieceLoc.y;
	bool whiteTurn = isWhite(board[x][y].piece);
	createStreakMove(board, heldPieceLoc, 1, 0);
	createStreakMove(board, heldPieceLoc, -1, 0);
	createStreakMove(board, heldPieceLoc, 0, 1);
	createStreakMove(board, heldPieceLoc, 0, -1);
}

void createKingMoves(struct tile board[8][8], struct specialMoves &sMoves, tileIndex heldPieceLoc){
	int x = heldPieceLoc.x; int y = heldPieceLoc.y;
	bool whiteTurn = isWhite(board[x][y].piece);
	//create 8 possible king moves
	createMove(board, x + 1, y + 1, ENEMY_OR_NOTHING, NORMAL_MOVE, whiteTurn);
	createMove(board, x, y + 1, ENEMY_OR_NOTHING, NORMAL_MOVE, whiteTurn);
	createMove(board, x - 1, y + 1, ENEMY_OR_NOTHING, NORMAL_MOVE, whiteTurn);
	createMove(board, x - 1, y, ENEMY_OR_NOTHING, NORMAL_MOVE, whiteTurn);
	createMove(board, x - 1, y - 1, ENEMY_OR_NOTHING, NORMAL_MOVE, whiteTurn);
	createMove(board, x, y - 1, ENEMY_OR_NOTHING, NORMAL_MOVE, whiteTurn);
	createMove(board, x + 1, y - 1, ENEMY_OR_NOTHING, NORMAL_MOVE, whiteTurn);
	createMove(board, x + 1, y, ENEMY_OR_NOTHING, NORMAL_MOVE, whiteTurn);
	//-------------------------SPECIAL MOVE: CASTLE
	setCastleMove(board, sMoves, whiteTurn);
}

void createQueenMoves(struct tile board[8][8], tileIndex heldPieceLoc){
	createBishopMoves(board, heldPieceLoc);
	createRookMoves(board, heldPieceLoc);
}

void createStreakMove(struct tile board[8][8], tileIndex heldPieceLoc, int xDisp, int yDisp){
	//true when streak hits a piece
	bool impact = false;
	int x = heldPieceLoc.x; int y = heldPieceLoc.y;
	int piece = board[x][y].piece;
	bool whiteTurn = isWhite(piece);
	x += xDisp; y += yDisp;
	//run streak until impact, updating valid moves
	while(isWithinBoard(x, y) && !impact){
		if(!isPiece(board[x][y].piece)){
			board[x][y].moveType = NORMAL_MOVE;
			x += xDisp; y += yDisp;
		}
		else{
			impact = true;
			piece = board[x][y].piece;
			if(isEnemyPiece(piece, whiteTurn)){
				board[x][y].moveType = NORMAL_MOVE;
			}
		}
	}	
}

void createMove(struct tile board[8][8], int x, int y, int prereqs, int moveType, bool whiteTurn){
	int piece = board[x][y].piece;
	if(isWithinBoard(x, y)){
		switch(prereqs){
			case ONLY_NOTHING:
				if(piece == NOTHING){
					board[x][y].moveType = moveType;
				}
				break;
			case ENEMY_OR_NOTHING:
				if(piece == NOTHING || isEnemyPiece(piece, whiteTurn)){
					board[x][y].moveType = moveType;
				}
				break;
			case ONLY_ENEMY:
				if(isEnemyPiece(piece, whiteTurn)){
					board[x][y].moveType = moveType;
				}
				break;
			default:
				break;
		}
	}
}

bool executeMove(struct tile board[8][8], struct specialMoves &sMoves, 
		tileIndex origin, tileIndex target, int &controlContext, bool &whiteTurn){
	bool returnVal = false;
	int tx = target.x; int ty = target.y;
	int ox = origin.x; int oy = origin.y;
	int moveType = board[tx][ty].moveType;
	int oPiece = board[ox][oy].piece;
	int dir = (whiteTurn) ? 1:-1;
	//clear enPassant placement.
	//If there should be a new enPassant, it will be updated in PAWN_JUMP switch case
	if(isValidMove(moveType)){
		//-------------------------game state update
		disableIneligibleCastleMoves(sMoves, oPiece, ox);
		clearEnPassant(sMoves.enPassant);
		//execute move
		switch(moveType){
			case NORMAL_MOVE:
		board[tx][ty].piece = board[ox][oy].piece;
		board[ox][oy].piece = NOTHING;
		controlContext = GAME_TURN;
		returnVal = true;
		break;
			case PAWN_JUMP:
		board[tx][ty].piece = board[ox][oy].piece;
		board[ox][oy].piece = NOTHING;
		setEnPassant(tx, ty, sMoves.enPassant);
		controlContext = GAME_TURN;
		returnVal = true;
		break;
			case EN_PASSANT:
		board[tx][ty].piece = board[ox][oy].piece;
		board[tx][ty - dir].piece = NOTHING;
		board[ox][oy].piece = NOTHING;
		controlContext = GAME_TURN;
		returnVal = true;
		break;
			case CASTLE:
		executeCastleMove(board, sMoves, tx, whiteTurn);
		controlContext = GAME_TURN;
		returnVal = true;
		break;
			case PROMOTION:
		//turn is swapped later after promotion choice in executePromotion function
		board[tx][ty].piece = board[ox][oy].piece;
		board[ox][oy].piece = NOTHING;
		controlContext = WAIT_PIECE_CHOICE;
		returnVal = false;
			default:
		break;
		}
	}
	return returnVal;
}

void setEnPassant(int movedToX, int movedToY, tileIndex &enPassant){
	int x = movedToX; int y = movedToY;
	//en passant is only created if unmoved pawn jumps to y index 3 or 4
	//en passant square set to tile behind jumping pawn
	if(y == 3){
		enPassant.x = x; enPassant.y = y - 1;
	}
	if(y == 4){
		enPassant.x = x; enPassant.y = y + 1;
	}
}

void clearEnPassant(tileIndex &enPassant){
	enPassant.x = NOTHING; enPassant.y = NOTHING;
}

void setCastleMove(struct tile board[8][8], struct specialMoves sMoves, bool whiteTurn){
	//all castle moves for a side if either rook moves
	int y = (whiteTurn) ? 0: 7;
	//king side castle
	if((whiteTurn && sMoves.WCastleValidKSide) || (!whiteTurn && sMoves.BCastleValidKSide)){
		if(board[1][y].piece == NOTHING && 
			board[2][y].piece == NOTHING){
			createMove(board, 1, y, ONLY_NOTHING, CASTLE, whiteTurn);
		}
	}
	//queen side castle
	if((whiteTurn && sMoves.WCastleValidQSide) || (!whiteTurn && sMoves.BCastleValidQSide)){
		if(board[4][y].piece == NOTHING && 
			board[5][y].piece == NOTHING &&
			board[6][y].piece == NOTHING){
			createMove(board, 5, y, ONLY_NOTHING, CASTLE, whiteTurn);
		}
	}
}

void executeCastleMove(struct tile board[8][8], struct specialMoves &sMoves, int x, bool whiteTurn){
	int y = (whiteTurn) ? 0: 7;
	int king;
	int rook;
	if(whiteTurn){
		king = WKING; rook = WROOK;
	}
	else{
		king = BKING; rook = BROOK;
	}
	switch(x){
		case 1:
			board[0][y].piece = NOTHING;
		   	board[3][y].piece = NOTHING;	
		   	board[1][y].piece = king;	
			board[2][y].piece = rook;
			break;
		case 5:
			board[7][y].piece = NOTHING;
		   	board[3][y].piece = NOTHING;	
		   	board[5][y].piece = king;	
			board[4][y].piece = rook;
			break;
		default:
			break;
	}
}

void disableIneligibleCastleMoves(struct specialMoves &sMoves, int oPiece, int ox){
	//if the king moves there are no more castle moves available for that side
	if(oPiece == WKING){
		sMoves.WCastleValidKSide = false;
		sMoves.WCastleValidQSide = false;
	}
	if(oPiece == BKING){
		sMoves.BCastleValidKSide = false;
		sMoves.BCastleValidQSide = false;
	}
	//if a rook moves only the castle move on that side is removed
	if(oPiece == WROOK){
		if(ox == 0){
			sMoves.WCastleValidKSide = false;
		}
		if(ox == 7){
			sMoves.WCastleValidQSide = false;
		}
	}	
	if(oPiece == BROOK){
		if(ox == 0){
			sMoves.BCastleValidKSide = false;
		}
		if(ox == 7){
			sMoves.BCastleValidQSide = false;
		}
	}	
}

void removeInvalidCastleMoves(struct tile board[8][8], struct specialMoves sMoves, bool whiteTurn){
	if(isInCheck(board, sMoves, whiteTurn)){
		for(int i = 0; i < 8; ++i){
			for(int j = 0; j < 8; ++j){
				if(board[i][j].moveType == CASTLE){
					board[i][j].moveType = CAN_NOT_CASTLE_IN_CHECK;
				}
			}
		}
	}
}

void executePromotion(struct tile board[8][8], struct specialMoves &sMoves, bool &whiteTurn, int choice){
	bool retVal = false;
	int promotionRow = (whiteTurn) ? 7 : 0;
	int queen = (whiteTurn) ? WQUEEN : BQUEEN; 
	int rook = (whiteTurn) ? WROOK : BROOK; 
	int bishop = (whiteTurn) ? WBISHOP : BBISHOP; 
	int knight = (whiteTurn) ? WKNIGHT : BKNIGHT; 
	int pawn = (whiteTurn) ? WPAWN : BPAWN;
	int colorChoice;
	switch(choice){
		case QUEEN:
			colorChoice = queen;
			break;
		case ROOK:
			colorChoice = rook;
			break;
		case BISHOP:
			colorChoice = bishop;
			break;
		case KNIGHT:
			colorChoice = knight;
			break;
	}
	//find and replace target pawn 
	for(int i = 0; i < 8; ++i){
		if(board[i][promotionRow].piece == pawn){
			board[i][promotionRow].piece = colorChoice;
			break;
		}
	}
}

void changeTurn(struct tile board[8][8], struct tile lastBoard[8][8], bool &whiteTurn){
	setLastMoveEmpty(board);
	updateLastMove(board, lastBoard);
	copyBoard(lastBoard, board);
	whiteTurn = !whiteTurn;
}

void updateLastMove(struct tile board[8][8], struct tile lastBoard[8][8]){
	for(int i = 0; i < 8; ++i){
		for(int j = 0; j < 8; ++j){
			if(board[i][j].piece != lastBoard[i][j].piece){
				board[i][j].lastMove = true;
			}
		}
	}
}

void setLastMoveEmpty(struct tile board[8][8]){
	for(int i = 0; i < 8; ++i){
		for(int j = 0; j < 8; ++j){
			board[i][j].lastMove = false;
		}
	}
}

