/*
Contributors: Liam T. Monson
License: BSD 2-Clause license which should be found in the "license.txt"
file in the root game directory.
*/
#ifndef GAME_LOGIC_H
#define GAME_LOGIC_H
#include "chessGDataStruct.hpp"
#include "networking.hpp"
#include "fileLoading.hpp"
void initChessGData(struct chessGData &GData);
void initBoard(struct tile board[8][8]);
tileIndex genTileIndex(int x, int y);
void initTileCoords(struct tileCoords tileLocs[8][8]);
void initNetPacks(chessGData &GData);
void initNetPackLog(chessGData &GData);
bool playerControlsSide(bool sideControl[2], bool whiteTurn);
bool isWithinBoard(int x, int y);
bool isPiece(int piece);
bool isHeldPiece(tileIndex heldPieceLoc, int x, int y);
bool isWhite(int piece);
bool isBlack(int piece);
bool isTurnPiece(int piece, bool whiteTurn);
bool isEnemyPiece(int piece, bool whiteTurn);
//whiteTurn is color whose king we're checking
bool isInCheck(struct tile board[8][8], struct specialMoves sMoves, bool whiteTurn);
//whiteTurn is color whose king we're checking
bool isCheckMate(struct tile board[8][8], struct specialMoves sMoves, int controlContext, bool whiteTurn);
bool isDraw(struct tile board[8][8], struct specialMoves sMoves, int controlContext, bool whiteTurn);
bool sideHasMoves(struct tile board[8][8], struct specialMoves sMoves, int controlContext, bool whiteTurn);
//returns true if indexed piece has any moves available
bool hasMoves(struct tile board[8][8], struct specialMoves sMoves, int controlContext, int x, int y);
//returns true if board[x][y].piece is within a valid move of an enemy piece
bool isInDanger(struct tile board[8][8], struct specialMoves sMoves, bool whiteTurn, int x, int y);
//true if move is valid, false if invalid | NOT_MOVE
bool isValidMove(int moveType);
//true if move is invalid, false if valid | NOT_MOVE
bool isInvalidMove(int moveType);
void setPlayerControl(bool sideControl[2], bool controlsWhite, bool controlsBlack);
void copyBoard(struct tile destBoard[8][8], struct tile sourceBoard[8][8]);
//uses a given line (row or column) from the given board and applies an offset
void shiftStrip(struct tile board[8][8], int offset, int tarLine, bool shiftRow);
void clearMoves(struct tile board[8][8]);
//assumes heldPieceLoc is a valid location and choice for the turn
void updateMoves(struct tile board[8][8], struct specialMoves sMoves,
	   	tileIndex heldPieceLoc, int controlContext);
void removeFriendlyCheckMoves(struct tile board[8][8], struct specialMoves sMoves,
		tileIndex heldPieceLoc, int controlContext);
void createPieceMoves(struct tile board[8][8], struct specialMoves &sMoves,
		tileIndex heldPieceLoc, int controlContext);
void createPawnMoves(struct tile board[8][8], tileIndex &enPassant, tileIndex heldPieceLoc);
void createKnightMoves(struct tile board[8][8], tileIndex heldPieceLoc);
void createBishopMoves(struct tile board[8][8], tileIndex heldPieceLoc);
void createRookMoves(struct tile board[8][8], tileIndex heldPieceLoc);
void createKingMoves(struct tile board[8][8], struct specialMoves &sMoves, tileIndex heldPieceLoc);
void createQueenMoves(struct tile board[8][8], tileIndex heldPieceLoc);
//apply valid moves in a line. xDisp and yDisp provide displacement
void createStreakMove(struct tile board[8][8], tileIndex heldPieceLoc, int xDisp, int yDisp);
enum CNMPrereqs{
	ONLY_NOTHING,
	ENEMY_OR_NOTHING,
	ONLY_ENEMY
};
//creates moveType at targetLoc if meets CNMPrereqs
void createMove(struct tile board[8][8], int x, int y, int prereqs, int moveType, bool whiteTurn);
//checks for valid move, moves piece and returns true if valid. returns false if invalid move
bool executeMove(struct tile board[8][8], struct specialMoves &sMoves, tileIndex origin, tileIndex target,
		int &controlContext, bool &whiteTurn);
void setEnPassant(int movedToX, int movedToY, tileIndex &enPassant);
void clearEnPassant(tileIndex &enPassant);
//x and y are coordinates of piece to determine what piece to add valid moves to
void setCastleMove(struct tile board[8][8], struct specialMoves sMoves, bool whiteTurn);
void executeCastleMove(struct tile board[8][8], struct specialMoves &sMoves, int x, bool whiteTurn);
//disables available special moves like K-side or Q-side castle 
void disableIneligibleCastleMoves(struct specialMoves &sMoves, int oPiece, int ox);
//removes castle moves if the king is in check
void removeInvalidCastleMoves(struct tile board[8][8], struct specialMoves sMoves, bool whiteTurn);
void executePromotion(struct tile board[8][8], struct specialMoves &sMoves, bool &whiteTurn, int choice);
//----Turn changing
//toggles whiteTurn and updates lastMoves of tiles
void changeTurn(struct tile board[8][8], struct tile lastBoard[8][8], bool &whiteTurn);
//checks for board piece differences and applies last move glow to them
void updateLastMove(struct tile board[8][8], struct tile lastBoard[8][8]);
//set all tiles on board to have false lastMove
void setLastMoveEmpty(struct tile board[8][8]);
#endif

