/*
Contributors: Liam T. Monson
License: BSD 2-Clause license which should be found in the "license.txt"
file in the root game directory.
*/
#include "input.hpp"
void handleMouseButton(GLFWwindow *window, int button, int action, int mods){
	//the server is not allowed to input game data unless the game is over
	if(GData.networkD.networkRole != SERVER || GData.controlContext == GAME_COMPLETION){
		networkPackage newNetPack;
		command newCommand;
		//setup newCommand
		newCommand.type = INVALID_INPUT;
		//give command correct target coordinates
		newCommand.target.x = GData.selectedTile.x;
		newCommand.target.y = GData.selectedTile.y;
		newCommand.choice = NOTHING;
		if(action == GLFW_PRESS){
			switch(button){
				case GLFW_MOUSE_BUTTON_1:
					if(!GData.inGameMenuOpen){
						switch(GData.controlContext){
							case MAIN_MENU:
							case GAME_TURN:
							case HOLDING_PIECE:
							case WAIT_SIDE_CHOICE:
							case WAIT_PIECE_CHOICE:
							case GAME_COMPLETION:
								newCommand.type = SELECT;
								initNetPack(GData, newNetPack, newCommand, COMMAND);
								GData.netPacks.push(newNetPack);
								break;
							default:
								break;
						}
					}
					//keep commands local if in-game menu is open
					else if(GData.inGameMenuOpen){
						switch(GData.controlContext){
							case MAIN_MENU:
							case GAME_TURN:
							case HOLDING_PIECE:
							case WAIT_SIDE_CHOICE:
							case WAIT_PIECE_CHOICE:
							case GAME_COMPLETION:
								newCommand.type = SELECT;
								initNetPack(GData, newNetPack, newCommand, LOCAL_ONLY);
								GData.netPacks.push(newNetPack);
								break;
							default:
								break;
						}
					}
					break;
				case GLFW_MOUSE_BUTTON_2:
					newCommand.type = DESELECT;
					initNetPack(GData, newNetPack, newCommand, COMMAND);
					GData.netPacks.push(newNetPack);
					break;
				default:
					break;
			}
		}
	}	
}

void handleKeys(GLFWwindow *window, int key, int scancode, int action, int mods){
	//the server is not allowed to input game data unless the game is over
	if(GData.networkD.networkRole != SERVER || GData.controlContext == GAME_COMPLETION){
		networkPackage newNetPack;
		command newCommand;
		newCommand.type = INVALID_INPUT;
		//give command out of bounds coordinates 
		newCommand.target.x = NOTHING;
		newCommand.target.y = NOTHING;
		newCommand.choice = NOTHING;
		if(action == GLFW_PRESS){
			//menu choices
			switch(GData.controlContext){
				case MAIN_MENU:
				case GAME_TURN:
				case HOLDING_PIECE:
				case WAIT_SIDE_CHOICE:
				case WAIT_PIECE_CHOICE:
				case GAME_COMPLETION:
					switch(key){
						case GLFW_KEY_ESCAPE:
							newCommand.type = KEYBOARD_PRESS;
							newCommand.choice = ESCAPE;
							initNetPack(GData, newNetPack, newCommand, LOCAL_ONLY);
							GData.netPacks.push(newNetPack);
							break;
						default:
							break;
					}
				default:
					break;
			}
			//other non-menu choices
			switch(GData.controlContext){
				case WAIT_SIDE_CHOICE:
					switch(key){
						case GLFW_KEY_1:
							newCommand.type = KEYBOARD_PRESS;
							newCommand.choice = WHITE_PLAYER;
							initNetPack(GData, newNetPack, newCommand, COMMAND);
							GData.netPacks.push(newNetPack);
							break;
						case GLFW_KEY_2:
							newCommand.type = KEYBOARD_PRESS;
							newCommand.choice = BLACK_PLAYER;
							initNetPack(GData, newNetPack, newCommand, COMMAND);
							GData.netPacks.push(newNetPack);
							break;
						default:
							break;
					}
					break;
				case WAIT_PIECE_CHOICE:
					switch(key){
						case GLFW_KEY_1:
							newCommand.type = KEYBOARD_PRESS;
							newCommand.choice = QUEEN;
							initNetPack(GData, newNetPack, newCommand, COMMAND);
							GData.netPacks.push(newNetPack);
							break;
						case GLFW_KEY_2:
							newCommand.type = KEYBOARD_PRESS;
							newCommand.choice = ROOK;
							initNetPack(GData, newNetPack, newCommand, COMMAND);
							GData.netPacks.push(newNetPack);
							break;
						case GLFW_KEY_3:
							newCommand.type = KEYBOARD_PRESS;
							newCommand.choice = BISHOP;
							initNetPack(GData, newNetPack, newCommand, COMMAND);
							GData.netPacks.push(newNetPack);
							break;
						case GLFW_KEY_4:
							newCommand.type = KEYBOARD_PRESS;
							newCommand.choice = KNIGHT;
							initNetPack(GData, newNetPack, newCommand, COMMAND);
							GData.netPacks.push(newNetPack);
							break;
					}
					break;
				case GAME_COMPLETION:
					break;
				default:
					break;
			}
		}
	}
}

