/*
Contributors: Liam T. Monson
License: BSD 2-Clause license which should be found in the "license.txt"
file in the root game directory.
*/
#ifndef NETWORKDATA_H
#define NETWORKDATA_H
#include <enet/enet.h>
#include <string>
#include "chessGameDataTypes.hpp"
#define MAX_SERVER_PEER_COUNT 8
#define NETWORK_TEXT_MESSAGE_CHARS 100

enum playerType{
	OBSERVER,
	WHITE_PLAYER,
	BLACK_PLAYER
};

struct playerData{
	ENetPeer *peerInfo;
	playerType playType;
	bool connected;
};

//forces server's state
struct enforcedState{
	int context;
};

enum packageType{
	ANNOUNCEMENT,
	COMMAND,
	FULL_GAME_UPDATE,
	//local only meaning don't send this data
	LOCAL_ONLY
};

struct networkPackage{
	packageType packType;
	enforcedState state;	
	command com;
	char textMessage[NETWORK_TEXT_MESSAGE_CHARS];
};

struct fullUpdatePackage{
	packageType packType;
	specialMoves sMoves;
	tileIndex heldPieceLoc;
	tile board[64];
	bool whiteTurn;
};

struct dataForClient{
	ENetAddress address;
	ENetHost *client;
	ENetPeer *peer;
	ENetEvent event;
	bool connectedToServer;
};

struct dataForServer{
	ENetAddress address;
	ENetHost *server;
	ENetEvent event;
	playerData playerD[MAX_SERVER_PEER_COUNT];
	int connectedHosts;
};

struct networkData{
	dataForClient clientData;
	dataForServer serverData;
	char textMessage[NETWORK_TEXT_MESSAGE_CHARS];
	int networkRole;
};
#endif
