/*
Contributors: Liam T. Monson
License: BSD 2-Clause license which should be found in the "license.txt"
file in the root game directory.
*/
#ifndef NETWORKING_H
#define NETWORKING_H
#include <enet/enet.h>
#include "networkData.hpp"
#include "mainChess.hpp"
#include <stdio.h>
#include <string.h>
#include <iostream>
//--------------------------------------------------general networking
void initNetworking(chessGData &GData, int &networkRole);
void deInitNetworking(chessGData &GData);
void updateNetworkCommands(chessGData &GData);
void initNetPack(chessGData &GData, networkPackage &newNetPack, command newCommand, packageType packType);
//--------------------------------------------------client
void initClient(dataForClient &clientData, std::string serverIP, enet_uint16 serverPort, int &networkRole);
void initClientAddress(ENetAddress &address, std::string serverIP, enet_uint16 serverPort);
void initClientHost(dataForClient &clientData);
void connectClientPeer(dataForClient &clientData);
void updateClientCommands(chessGData &GData, dataForClient &clientData);
void updateNetworkTextMessage(networkData &networkD, networkPackage package);
void processClientPackages(chessGData &GData, dataForClient &clientData);
void processFullUpdatePackage(chessGData &GData, dataForClient &clientData);
void sendClientPackages(chessGData &GData, dataForClient &clientData);
//--------------------------------------------------server
void initServer(dataForServer &serverData, enet_uint16 serverPort, int &networkRole);
void initServerAddress(ENetAddress &address, enet_uint16 serverPort);
void initServerHost(dataForServer &serverD);
void updateServerCommands(chessGData &GData, dataForServer &serverData);
//if a player disconnects make the game wait for the player slots to fill
void handlePlayerSlots(chessGData &GData, dataForServer &serverData);
void processServerPackages(chessGData &GData, dataForServer &serverData);
void initPlayerData(playerData &playerD);
void setPlayerData(playerData &playerD, ENetPeer *peerInfo, playerType type, bool connected);
void setConnectPeerData(chessGData &GData, dataForServer &serverData);
void setDisconnectPeerData(dataForServer &serverData);
void setNetworkTextMessage(networkPackage &package, playerType type);
void setServerStateEnforcement(chessGData GData, networkPackage &package);
bool bothPlayerSlotsOccupied(dataForServer &serverData);
bool whitePlayerSlotOccupied(dataForServer &serverData);
bool blackPlayerSlotOccupied(dataForServer &serverData);
bool appropriateCommandAuthority(dataForServer &serverData, int controlContext);
void broadcastPlayerSlotChoices(chessGData GData, dataForServer &serverData);
void broadcastServerAnnouncement(chessGData GData, dataForServer &serverData, int controlContext);
void sendFullGameUpdate(chessGData &GData, dataForServer &serverData);
void swapControlContextWaitForPlayers(int &controlContext);
#endif

